#!/bin/bash

function error() {
	echo "${0}: error: ${*}" >&2
	exit 1
}

unset GPS_START GPS_END TRIG_START TRIG_END DAGFILE
while getopts "a:b:e:f:s:t:" OPT ; do
	case "${OPT}" in
	# GPS start time
	s)
		GPS_START="${OPTARG}" ;;
	
	# GPS end time
	e)
		GPS_END="${OPTARG}" ;;
	
	# GPS trigger start time
	a)
		TRIG_START="${OPTARG}" ;;
	
	# GPS trigger end time
	b)
		TRIG_END="${OPTARG}" ;;
	
	# DAG filename
	f)
		DAGFILE="${OPTARG}" ;;
	
	# OnASys run-time configuration top-dir
	t)
		DATADIR="${OPTARG}" ;;
	esac
done

unset GPS_DUR TRIG_DUR
GPS_DUR=$((${GPS_END} - ${GPS_START}))
TRIG_DUR=$((${TRIG_END} - ${TRIG_START}))

#
# Copy .sub files
#

cp "${DATADIR}"/*.sub . || error "cannot copy \"${DATADIR}\"/*.sub to ."

#
# Copy .dag file, performing search-and-replace on start and end times.
# Note:  it is not an error for there to be more than one template .dag
# file in the data directory, but the effect will be to have them all
# concatenated together into the concrete DAG.  This may or may not be what
# you want to have happen, so plan accordingly.
#

cat "${DATADIR}"/*.dag | sed -e ": loop" -e "s?GPSSTARTTIME?${GPS_START}?" -e "s?GPSENDTIME?${GPS_END}?" -e "s?GPSDURATION?${GPS_DUR}?" -e "s?TRIGSTARTTIME?${TRIG_START}?" -e "s?TRIGENDTIME?${TRIG_END}?" -e "s?TRIGDURATION?${TRIG_DUR}?" -e "t loop" >"${DAGFILE}" || error "cannot copy \"${DATADIR}\"/*.dag to \"${DAGFILE}\""

#
# Create the subdirectories
#

mkdir logs cache || error "cannot create subdirectories logs and cache"

