
from sqlobject import *
import mx.DateTime

# This is awfully hacky.
# Importing BOSS simply to get the connection string.
# For our (current) purposes, BOSS *is* where this lives, but
# it might not always be.  Curse expedience!
try:
    import BOSS
    conn = BOSS.config['conn']
except:
    conn = None


class DagNode(SQLObject):
    _connection = conn
    _table = "DAGNODE"
    jobid = StringCol(length=100, notNone=1)
    name = StringCol(length=100, notNone=1)
    t_sub = DateTimeCol(default=0)
    t_start = DateTimeCol(default=0)
    t_stop = DateTimeCol(default=0)
    t_stat = StringCol(length=100, default='U')
    ret_code = StringCol(length=11, default=None)
    stderr = StringCol(length=100, default=None)
    stdout = StringCol(length=100, default=None)
    subfile = StringCol(length=100, notNone=1)
    children = RelatedJoin('DagNode', joinColumn='pid', otherColumn='cid',
                           intermediateTable='DAGARC', addRemoveName='Child',
                           joinMethodName='children')
    parents = RelatedJoin('DagNode', joinColumn='cid', otherColumn='pid', 
                           intermediateTable='DAGARC', addRemoveName='Parent',
                           joinMethodName='parents')

#    # the following don't seem to do squat.
#    def _set_t_sub(self, value):
#        if type(value) == str:
#            print "Setting Converted Date", value
#            self._SO_set_t_sub(mx.DateTime.DateTimeFrom(value))
#        else:
#            print "Setting RAW Date", value
#            self._SO_set_t_sub(value)
#
#    def _set_t_start(self, value):
#        if type(value) == str:
#            self._SO_set_t_start(mx.DateTime.DateTimeFrom(value))
#        else:
#            self._SO_set_t_start(value)
#
#    def _set_t_stop(self, value):
#        if type(value) == str:
#            self._SO_set_t_stop(mx.DateTime.DateTimeFrom(value))
#        else:
#            self._SO_set_t_stop(value)

import re

def setNodeOutfilesFromSub(node, subfile):
    # Get the 'output' and 'error' files from a .sub file for a node.
    out = None
    err = None
    outputpat = re.compile(r'OUTPUT\s*=\s*(?P<fname>.*)',re.I)
    errorpat = re.compile(r'ERROR\s*=\s*(?P<fname>.*)',re.I)

    print "Looking for output files for node:", node
    f = open(subfile,"r")
    if not f:
        print "Trouble opening subfile", subfile, "for node", node
        return
    for line in f.readlines():
        m = outputpat.search(line)
        if m:
            node.stdout = m.group('fname')
            print "Found stdout:", node.stdout
        m = errorpat.search(line)
        if m:
            node.stderr = m.group('fname')
            print "Found stderr:", node.stderr


class DAG(object):

    def __init__(self, jobid, spec, _USE_NODES=None):
        self.jobid = jobid
        if _USE_NODES != None:
            self._node = _USE_NODES
            return
        self._node = {}
        if type(spec) == str:
            lines = spec.split('\n')
        elif type(spec) == file:
            lines = spec.readlines()
        else:
            raise Exception("DAG cannot initialize using spec: %s"%spec)

        jobpat = re.compile(r'^JOB\s+(?P<nodeid>\w+)\s+(?P<subfile>.*)',re.I)
        arcpat = re.compile(r'^PARENT\s+(?P<parents>.+?)\s+CHILD\s+(?P<children>.+)',re.I)
        for line in lines:
            # XXX debug
            #print "Scanning dag file line:", line
            m = jobpat.search(line)
            if m:
                name = m.group('nodeid')
                subfile = m.group('subfile')
                # XXX debug
                print 'New Node', name
                n = DagNode(jobid=self.jobid, name=name, subfile=subfile)
                self._node[name] = n
                setNodeOutfilesFromSub(n, subfile)
            m = arcpat.search(line)
            if m:
                self._registerArcs(m.group('parents'), m.group('children'))

    def _registerArcs(self, ps, cs):
        parents = ps.split(' ')
        children = cs.split(' ')
        parents = [p.strip() for p in parents]
        children = [c.strip() for c in children]

        for p in parents:
            for c in children:
                par = DagNode.select(AND(DagNode.q.jobid==self.jobid, \
                                         DagNode.q.name==p))[0]
                kid = DagNode.select(AND(DagNode.q.jobid==self.jobid, \
                                         DagNode.q.name==c))[0]
                par.addChild(kid)
                # XXX debug
                print 'New Arc', par.name, kid.name

    def node(self, name):
        try:
            return self._node[name]
        except:
            return None

    def nodeNames(self):
        return self._node.keys()

    def nodes(self):
        return self._node.values()

    def get(cls, jobid):
        nodes = DagNode.select(DagNode.q.jobid==jobid)
        if nodes.count() == 0:
            # There is no dag with this jobid.
            return None
        n = {}
        for node in nodes:
            n[node.name] = node
        return cls(jobid=jobid, spec=None, _USE_NODES=n)

    get= classmethod(get)

